// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: https://codemirror.net/LICENSE
// livecodescript.js by Hermann Hoch, based on dylan.js

(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    mod(require("../../lib/codemirror"));
  else if (typeof define == "function" && define.amd) // AMD
    define(["../../lib/codemirror"], mod);
  else // Plain browser env
    mod(CodeMirror);
})(function(CodeMirror) {
"use strict";

function forEach(arr, f) {
  for (var i = 0; i < arr.length; i++) f(arr[i], i)
}
function some(arr, f) {
  for (var i = 0; i < arr.length; i++) if (f(arr[i], i)) return true
  return false
}

CodeMirror.defineMode("livecodescript", function(_config) {
  // Words
  var words = {
    // Words that introduce unnamed definitions like "define interface"
    unnamedDefinition: ["_hscrollbarid", "_ideoverride", "_unplacedgroupids", "_vscrollbarid", "abbr", "abbrev", "abbreviated", "acceleratedrendering", "acceleratorkey", "acceleratormodifiers", "acceleratortext", "accelkey", "accelmods", "acceltext", "accentcolor", "acceptdrop", "activatepalettes", "address", "allowabledragactions", "allowdatagrambroadcasts", "allowfieldredraw", "allowinlineinput", "allowinterrupts", "allowkeyinfield", "alphadata", "altid", "alwaysbuffer", "angle", "antialiased", "arcangle", "arm", "armborder", "armed", "armedicon", "armfill", "arrowsize", "autoarm", "autohilight", "autohilite", "autoselect", "autotab", "availableprinters", "backcolor", "backdrop", "backgroundbehavior", "backgroundcolor", "backgroundids", "backgroundnames", "backgroundpattern", "backgroundpixel", "backlist", "backpattern", "backpixel", "backsize", "beepduration", "beeploudness", "beeppitch", "beepsound", "behavior", "bgbehavior", "blendlevel", "blindtyping", "blinkrate", "bordercolor", "borderpattern", "borderwidth", "botleft", "botright", "bottom", "bottomcolor", "bottomleft", "bottommargin", "bottompattern", "bottompixel", "bottomright", "boundingbox", "boundingrect", "breakpoints", "brush", "brushcolor", "brushpattern", "bufferhiddenimages", "buffermode", "callbacks", "cantabort", "cantdelete", "cantmodify", "cantselect", "capstyle", "cardids", "cardnames", "casesensitive", "centered", "centerrect", "centerrectangle", "centurycutoff", "charindex", "charset", "checkmark", "childcontrolids", "childcontrolnames", "clipboarddata", "clipstorect", "closebox", "collapsebox", "colordialogcolors", "colormap", "coloroverlay", "colors", "colorworld", "columndel", "columndelimiter", "commandchar", "compositorcachelimit", "compositorstatistics", "compositortilesize", "compositortype", "constantmask", "constraints", "controlids", "controlnames", "convertoctals", "currentcard", "currentframe", "currentnode", "currentpage", "currenttime", "cursor", "cursorMovement", "customkeys", "customproperties", "custompropertyset", "custompropertysets", "dashes", "debugcontext", "decorations", "default", "defaultbutton", "defaultcursor", "defaultfolder", "defaultmenubar", "defaultnetworkinterface", "defaultstack", "deferscreenupdates", "desktop", "destroystack", "destroywindow", "detailed", "dialogdata", "directory", "disabled", "disabledicon", "documentfilename", "documents", "dontdither", "dontrefresh", "dontresize", "dontsearch", "dontusens", "dontuseqt", "dontuseqteffects", "dontwrap", "doubleclickdelta", "doubleclickinterval", "dragaction", "dragdata", "dragdelta", "draggable", "dragimage", "dragimageoffset", "dragspeed", "dropshadow", "duration", "dynamicpaths", "editbackground", "editbg", "editbkgnd", "editiontype", "editmenus", "editmode", "editscripts", "effective", "effectrate", "eighthcolor", "eighthpixel", "emacskeybindings", "enabled", "enabledtracks", "encoding", "endarrow", "endframe", "endtime", "endvalue", "engine", "english", "eraser", "errormode", "executioncontexts", "explicitvariables", "explicitvars", "extendkey", "externalcommands", "externalfunctions", "externalpackages", "externals", "family", "fifthcolor", "fifthpixel", "filename", "filetype", "fillback", "filled", "fillfore", "fillgradient", "fillpat", "fillrule", "firstcolor", "firstindent", "firstpixel", "fixedlineheight", "flagged", "flaggedranges", "focuscolor", "focuspattern", "focuspixel", "fontfilesinuse", "forecolor", "foregroundcolor", "foregroundpattern", "foregroundpixel", "forepattern", "forepixel", "formatforprinting", "formattedheight", "formattedleft", "formattedrect", "formattedstyledtext", "formattedtext", "formattedtop", "formattedwidth", "formsensitive", "fourthcolor", "fourthpixel", "framecount", "framerate", "freesize", "ftpproxy", "fullclipboarddata", "fulldragdata", "fullscreen", "fullscreenmode", "grid", "gridsize", "groupids", "groupnames", "hcaddressing", "hcimportstat", "hcstack", "height", "hgrid", "hidden", "hidebackdrop", "hideconsolewindows", "hidepalettes", "highlight", "highlighted", "highlite", "highlited", "hilight", "hilighted", "hilite", "hiliteborder", "hilitecolor", "hilited", "hilitedbutton", "hilitedbuttonid", "hilitedbuttonname", "hilitedicon", "hilitedline", "hilitedlines", "hilitefill", "hiliteicon", "hilitepattern", "hilitepixel", "home", "hotspot", "hotspots", "hovericon", "hscroll", "hscrollbar", "htmltext", "httpheaders", "httpproxy", "icon", "icongravity", "iconic", "iconmenu", "id", "idlerate", "idleticks", "ignoremouseevents", "imagecachelimit", "imagecacheusage", "imagedata", "imagepixmapid", "imagesource", "ink", "innerglow", "innershadow", "internet", "inv", "invisible", "itemdel", "itemdelimiter", "joinstyle", "jpegquality", "kind", "label", "labelwidth", "layer", "layermode", "left", "leftindent", "leftmargin", "libraries", "linedel", "linedelimiter", "lineinc", "lineincrement", "lineindex", "linesize", "linkcolor", "linkhilitecolor", "linktext", "linkvisitedcolor", "listbehavior", "listdepth", "listindent", "listindex", "liststyle", "liveresizing", "loadedextensions", "loadedtime", "loc", "location", "lockcolormap", "lockcursor", "locked", "lockerrordialogs", "lockloc", "locklocation", "lockmenus", "lockmessages", "lockmoves", "lockrecent", "lockscreen", "locktext", "lockupdates", "long", "longwindowtitles", "lookandfeel", "looping", "lowresolutiontimers", "lzwkey", "magnify", "mainstack", "margins", "mark", "markchar", "markercolor", "markerdrawn", "markerfillcolor", "markerfilled", "markerlinesize", "markerpattern", "markerpoints", "maskdata", "maskpixmapid", "maxheight", "maximizebox", "maxwidth", "mediatypes", "menubar", "menuhistory", "menulines", "menumessage", "menumode", "menumousebutton", "menuname", "messagemessages", "metadata", "metal", "minheight", "minimizebox", "minstackfileversion", "minwidth", "mirrored", "miterlimit", "mnemonic", "mode", "modifiedmark", "movespeed", "moviecontrollerid", "multieffect", "multiple", "multiplehilites", "multiplelines", "multispace", "name", "navigationarrows", "networkinterfaces", "nodes", "noncontiguoushilites", "num", "number", "numberformat", "opaque", "orientation", "outerglow", "outputlineendings", "outputtextencoding", "padding", "pagecount", "pageheights", "pageinc", "pageincrement", "pageranges", "paintcompression", "palindromeframes", "pan", "parentscript", "partnumber", "passkey", "password", "pattern", "patterns", "paused", "penback", "pencolor", "penfore", "penheight", "penpat", "penpattern", "penwidth", "pixelscale", "pixmapid", "plaintext", "playdestination", "playloudness", "playrate", "playselection", "pointerfocus", "points", "polysides", "postscript", "powerkeys", "preservevariables", "preservevars", "printcardborders", "printcollate", "printcolors", "printcommand", "printcopies", "printduplex", "printerfeatures", "printername", "printeroutput", "printersettings", "printfonttable", "printgutters", "printmargins", "printpagenumber", "printpaperorientation", "printpaperrect", "printpaperrectangle", "printpaperscale", "printpapersize", "printranges", "printrect", "printrectangle", "printrotated", "printrowsfirst", "printscale", "printtextalign", "printtextfont", "printtextheight", "printtextsize", "printtextstyle", "printtitle", "privatecolors", "processtype", "prolog", "properties", "proportionalthumbs", "qtidlerate", "radiobehavior", "raisemenus", "raisepalettes", "raisewindows", "randomseed", "rawclipboarddata", "rawdragdata", "recentcards", "recentnames", "recordchannels", "recordcompression", "recordformat", "recording", "recordinput", "recordrate", "recordsamplesize", "rect", "rectangle", "recursionlimit", "referringstack", "relativepoints", "relayergroupedcontrols", "remapcolor", "repeatcount", "repeatdelay", "repeatrate", "resizable", "resizequality", "retainimage", "retainpostscript", "revavailablehandlers", "revavailablevariables", "revcrashreportsettings", "revlicenseinfo", "revlicenselimits", "revmessageboxlastobject", "revmessageboxredirect", "revobjectlisteners", "revpropertylistenerthrottle", "revruntimebehaviour", "revunplacedgroupids", "right", "rightindent", "rightmargin", "roundends", "roundheight", "roundradius", "roundwidth", "rowdel", "rowdelimiter", "rtftext", "savecompressed", "scale", "scalefactor", "scaleindependently", "screen", "screengamma", "screenmouseloc", "screennopixmaps", "screenpixelscale", "screenpixelscales", "screensharedmemory", "screenvcsharedmemory", "script", "scriptexecutionerrors", "scriptonly", "scriptparsingerrors", "scriptstatus", "scripttextfont", "scripttextsize", "scroll", "scrollbarwidth", "secondcolor", "secondpixel", "securemode", "securitycategories", "securitypermissions", "selected", "selectedcolor", "selectgroupedcontrols", "selectionhandlecolor", "selectionmode", "serialcontrolstring", "sessioncookiename", "sessionid", "sessionlifetime", "sessionsavepath", "seventhcolor", "seventhpixel", "shadow", "shadowcolor", "shadowoffset", "shadowpattern", "shadowpixel", "sharedbehavior", "sharedgroupids", "sharedgroupnames", "sharedhilite", "sharedtext", "shellcommand", "short", "showbadge", "showborder", "showcontroller", "showfill", "showfocusborder", "showgroups", "showhilite", "showicon", "showinvisibles", "showlines", "showname", "showpen", "showpict", "showselection", "showvalue", "sixthcolor", "sixthpixel", "size", "slices", "sockettimeoutinterval", "soundchannel", "spaceabove", "spacebelow", "spray", "sslcertificates", "stackfiles", "stackfiletype", "stackfileversion", "stacklimit", "stacksinuse", "startangle", "startarrow", "startframe", "starttime", "startupiconic", "startvalue", "status", "statusicon", "statusiconmenu", "statusicontooltip", "strokegradient", "style", "styledtext", "substacks", "syncrate", "system", "systemcolorselector", "systemfileselector", "systempixelscale", "systemprintselector", "systemwindow", "tabalign", "tabgroupbehavior", "tabstops", "tabwidths", "temporary", "text", "textalign", "textarrows", "textcolor", "textdata", "textdirection", "textfont", "textheight", "textpattern", "textshift", "textsize", "textstyle", "theme", "themeclass", "thirdcolor", "thirdpixel", "threed", "threedhilite", "thumbcolor", "thumbpattern", "thumbpos", "thumbposition", "thumbsize", "tilt", "timescale", "title", "titlewidth", "togglehilites", "tool", "tooltip", "tooltipdelay", "top", "topcolor", "topleft", "topmargin", "toppattern", "toppixel", "topright", "traceabort", "tracedelay", "tracereturn", "tracestack", "traceuntil", "trackcount", "tracks", "traversalon", "twelvehourtime", "typingrate", "umask", "unboundedhscroll", "unboundedvscroll", "underlinelinks", "unicodeacceleratortext", "unicodeformattedtext", "unicodelabel", "unicodeplaintext", "unicodetext", "unicodetitle", "unicodetooltip", "urlresponse", "usepixelscaling", "userlevel", "usermodify", "userproperties", "usesystemdate", "useunicode", "vcplayer", "vgrid", "videoclipplayer", "vis", "visible", "visited", "visitedicon", "vscroll", "vscrollbar", "watchedvariables", "wholematches", "widemargins", "width", "windowboundingrect", "windowid", "windowmanagerplace", "windowshape", "wmplace", "working", "xextent", "xhot", "xoffset", "xscale", "yextent", "yhot", "yoffset", "yscale", "zoom", "zoombox" ],

    // Words that introduce simple named definitions like "define library"
    namedDefinition: [],

    // Words that introduce type definitions like "define class".
    // These are also parameterized like "define method" and are
    // appended to otherParameterizedDefinitionWords
    typeParameterizedDefinition: ["_internal", "accept", "add", "answer", "ask", "assert", "beep", "break", "breakpoint", "call", "cancel", "choose", "clear", "click", "clone", "close", "combine", "compact", "constant", "convert", "copy", "create", "crop", "cut", "debugdo", "decrypt", "define", "dehilite", "delete", "disable", "dispatch", "divide", "do", "domenu", "drag", "drawer", "edit", "enable", "encrypt", "exit", "export", "filter", "find", "flip", "focus", "get", "global", "go", "grab", "hide","import", "include", "insert", "intersect", "kill", "launch", "library", "load", "local", "lock", "modal", "modeless", "move", "multiply", "new", "next", "open", "option", "palette", "pass", "paste", "place", "play", "pop", "popup", "post", "prepare", "print", "pulldown", "push", "put", "quit", "read", "record", "redo", "relayer", "release", "remove", "rename", "replace", "reply", "request", "require", "reset", "resolve", "return", "revert", "revrelicense", "rotate", "save", "secure", "seek", "select", "send", "set", "sheet", "show", "sort", "split", "start", "stop", "subtract", "throw", "toplevel", "type", "undefine", "undo", "ungroup", "unhilite", "union", "unload", "unlock", "unmark", "visual", "wait", "write" ],

    // Words that introduce trickier definitions like "define method".
    // These require special definitions to be added to startExpressions
    otherParameterizedDefinition: [],

    // Words that introduce module constant definitions.
    // These must also be simple definitions and are
    // appended to otherSimpleDefinitionWords
    constantSimpleDefinition: ["arrow", "backslash", "busy", "clock", "colon", "comma", "cr", "crlf", "cross", "done", "down", "eight", "empty", "endoffile", "eof", "false", "five", "formfeed", "four", "hand", "help", "ibeam", "lf", "linefeed", "nine", "none", "null", "one", "pi", "plus", "quote", "scrollbarfactor", "seven", "six", "slash", "space", "tab", "ten", "three", "true", "two", "up", "watch", "zero" ],

    // Words that introduce module variable definitions.
    // These must also be simple definitions and are
    // appended to otherSimpleDefinitionWords
    variableSimpleDefinition: ["abs", "acos", "aliasreference", "alternatelanguages", "altkey", "annuity", "arithmeticmean", "arraydecode", "arrayencode", "asin", "atan", "atan2", "average", "averagedeviation", "avg", "avgdev", "backscripts", "base64decode", "base64encode", "baseconvert", "bididirection", "binarydecode", "binaryencode", "buildnumber", "byteoffset", "bytetonum", "cachedurl", "cachedurls", "capslockkey", "ceil", "ceiling", "chartonum", "ciphernames", "clickchar", "clickcharchunk", "clickchunk", "clickfield", "clickh", "clickline", "clickloc", "clickstack", "clicktext", "clickv", "clipboard", "cmdargs", "cmdkey", "cmdname", "codepointoffset", "codepointproperty", "codepointtonum", "codeunitoffset", "colornames", "commandarguments", "commandkey", "commandname", "commandnames", "compound", "compress", "constantnames", "controlatloc", "controlatscreenloc", "controlkey", "copyresource", "cos", "ctrlkey", "currentwindow", "date", "dateformat", "decompress", "deleteregistry", "deleteresource", "directories", "diskspace", "dnsservers", "dragdestination", "dragsource", "drivernames", "drives", "dropchunk", "environment", "eventaltkey", "eventcapslockkey", "eventcommandkey", "eventcontrolkey", "eventoptionkey", "eventshiftkey", "existence", "exists", "exp", "exp1", "exp10", "exp2", "extents", "files", "floor", "flushevents", "focusedobject", "folders", "fontlanguage", "fontnames", "fontsizes", "fontstyles", "format", "foundchunk", "foundfield", "foundline", "foundloc", "foundtext", "frontscripts", "functionnames", "geometricmean", "getresource", "getresources", "globalloc", "globalnames", "globals", "harmonicmean", "hasmemory", "heapspace", "hilitedtext", "hostaddress", "hostaddresstoname", "hostname", "hostnametoaddress", "httpproxyforurl", "interrupt", "isnumber", "isotomac", "itemoffset", "keys", "keysdown", "len", "length", "licensed", "lineoffset", "listregistry", "ln", "ln1", "localloc", "localnames", "log10", "log2", "longfilepath", "lower", "machine", "mactoiso", "mainstacks", "matchchunk", "matchtext", "matrixmultiply", "max", "mcencrypt", "mcisendstring", "md5digest", "me", "mean", "measuretext", "measureunicodetext", "median", "menubutton", "menuobject", "menus", "merge", "millisec", "millisecond", "milliseconds", "millisecs", "min", "monthnames", "mouse", "mousechar", "mousecharchunk", "mousechunk", "mouseclick", "mousecolor", "mousecontrol", "mouseh", "mouseline", "mouseloc", "mousestack", "mousetext", "mousev", "movie", "movies", "movingcontrols", "nativechartonum", "normalizetext", "numtobyte", "numtochar", "numtocodepoint", "numtonativechar", "offset", "openfiles", "openprocesses", "openprocessids", "opensockets", "openstacks", "optionkey", "owner", "paragraphoffset", "param", "paramcount", "params", "peeraddress", "pendingmessages", "platform", "popstddev", "populationstandarddeviation", "populationvariance", "popvariance", "processid", "processor", "propertynames", "qteffects", "qtversion", "queryregistry", "random", "randombytes", "recordcompressiontypes", "recordloudness", "replacetext", "result", "round", "samplestandarddeviation", "samplevariance", "sampstddev", "sampvariance", "screencolors", "screendepth", "screenloc", "screenname", "screenrect", "screenrects", "screentype", "screenvendor", "scriptlimits", "sec", "seconds", "secs", "segmentoffset", "selectedbutton", "selectedchunk", "selectedfield", "selectedgraphic", "selectedimage", "selectedline", "selectedlines", "selectedloc", "selectedobject", "selectedobjects", "selectedtext", "selection", "selobj", "selobjs", "sentenceoffset", "setregistry", "setresource", "sha1digest", "shell", "shiftkey", "shortfilepath", "sin", "sound", "specialfolderpath", "sqrt", "stacks", "stackspace", "standarddeviation", "statround", "stddev", "sum", "syserror", "systemversion", "tan", "target", "templateaudioclip", "templatebutton", "templatecard", "templateeps", "templatefield", "templategraphic", "templategroup", "templateimage", "templateplayer", "templatescrollbar", "templatestack", "templatevideoclip", "tempname", "textdecode", "textencode", "textheightsum", "tick", "ticks", "time", "tokenoffset", "tolower", "topstack", "topwindow", "toupper", "transpose", "truewordoffset", "trunc", "unidecode", "uniencode", "upper", "urldecode", "urlencode", "urlstatus", "uuid", "value", "variablenames", "variance", "vectordotproduct", "version", "volumes", "waitdepth", "weekdaynames", "windows", "wordoffset" ],

    // Other words that introduce simple definitions
    // (without implicit bodies).
    otherSimpleDefinition: [],

    // Words that begin statements with implicit bodies.
    statement: ["after", "before", "case", "catch", "command", "else", "end", "for", "function", "getprop", "if", "into", "of", "on", "private", "repeat", "setprop", "switch", "the", "then", "to", "try", "while", "with"],

    // Patterns that act as separators in compound statements.
    // This may include any general pattern that must be indented
    // specially.
    separator: [],

    // Keywords that do not require special indentation handling,
    // but which should be highlighted
    other: ["a", "among", "and", "audioclip", "bitAnd", "bitNot", "bitOr", "bitXor", "boolean", "button", "card", "char", "character", "color", "contains", "div", "field", "file", "folder", "graphic", "group", "image", "in", "integer", "is", "item", "line", "mod", "no", "not", "or", "player", "scrollbar", "stack", "string", "there", "token", "videoclip", "within", "word" ],

    // Condition signaling function calls
    signalingCalls: []
  };

  words["otherDefinition"] =
    words["unnamedDefinition"]
    .concat(words["namedDefinition"])
    .concat(words["otherParameterizedDefinition"]);

  words["definition"] =
    words["typeParameterizedDefinition"]
    .concat(words["otherDefinition"]);

  words["parameterizedDefinition"] =
    words["typeParameterizedDefinition"]
    .concat(words["otherParameterizedDefinition"]);

  words["simpleDefinition"] =
    words["constantSimpleDefinition"]
    .concat(words["variableSimpleDefinition"])
    .concat(words["otherSimpleDefinition"]);

  words["keyword"] =
    words["statement"]
    .concat(words["separator"])
    .concat(words["other"]);

  // Patterns
  var symbolPattern = "[-_a-zA-Z?!*@<>$%]+";
  var symbol = new RegExp("^" + symbolPattern);
  var patterns = {
    // Symbols with special syntax
    symbolKeyword: symbolPattern + ":",
    symbolClass: "<" + symbolPattern + ">",
    symbolGlobal: "\\*" + symbolPattern + "\\*",
    symbolConstant: "\\$" + symbolPattern
  };
  var patternStyles = {
    symbolKeyword: "atom",
    symbolClass: "tag",
    symbolGlobal: "variable-2",
    symbolConstant: "variable-3"
  };

  // Compile all patterns to regular expressions
  for (var patternName in patterns)
    if (patterns.hasOwnProperty(patternName))
      patterns[patternName] = new RegExp("^" + patterns[patternName]);

  // Names beginning "with-" and "without-" are commonly
  // used as statement macro
  patterns["keyword"] = [/^with(?:out)?-[-_a-zA-Z?!*@<>$%]+/];

  var styles = {};
  styles["keyword"] = "keyword";
  styles["definition"] = "def";
  styles["simpleDefinition"] = "def";
  styles["signalingCalls"] = "builtin";

  // protected words lookup table
  var wordLookup = {};
  var styleLookup = {};

  forEach([
    "keyword",
    "definition",
    "simpleDefinition",
    "signalingCalls"
  ], function(type) {
    forEach(words[type], function(word) {
      wordLookup[word] = type;
      styleLookup[word] = styles[type];
    });
  });


  function chain(stream, state, f) {
    state.tokenize = f;
    return f(stream, state);
  }

  function tokenBase(stream, state) {
    // String
    var ch = stream.peek();
    if (ch == "'" || ch == '"') {
      stream.next();
      return chain(stream, state, tokenString(ch, "string"));
    }
    // Comment
    else if (ch == "/") {
      stream.next();
      if (stream.eat("*")) {
        return chain(stream, state, tokenComment);
      } else if (stream.eat("/")) {
        stream.skipToEnd();
        return "comment";
      }
      stream.backUp(1);
    }
    else if (ch == "-") {
      stream.next();
      if (stream.eat("-")) {
        stream.skipToEnd();
        return "comment";
      }
    }
    else if (ch == "#") {
      stream.skipToEnd();
      return "comment";
    }
    // Decimal
    else if (/[+\-\d\.]/.test(ch)) {
      if (stream.match(/^[+-]?[0-9]*\.[0-9]*([esdx][+-]?[0-9]+)?/i) ||
          stream.match(/^[+-]?[0-9]+([esdx][+-]?[0-9]+)/i) ||
          stream.match(/^[+-]?\d+/)) {
        return "number";
      }
    }
    // Hash
    else if (ch == "#") {
      stream.next();
      // Symbol with string syntax
      ch = stream.peek();
      if (ch == '"') {
        stream.next();
        return chain(stream, state, tokenString('"', "string"));
      }
      // Binary number
      else if (ch == "b") {
        stream.next();
        stream.eatWhile(/[01]/);
        return "number";
      }
      // Hex number
      else if (ch == "x") {
        stream.next();
        stream.eatWhile(/[\da-f]/i);
        return "number";
      }
      // Octal number
      else if (ch == "o") {
        stream.next();
        stream.eatWhile(/[0-7]/);
        return "number";
      }
      // Token concatenation in macros
      else if (ch == '#') {
        stream.next();
        return "punctuation";
      }
      // Sequence literals
      else if ((ch == '[') || (ch == '(')) {
        stream.next();
        return "bracket";
      // Hash symbol
      } else if (stream.match(/f|t|all-keys|include|key|next|rest/i)) {
        return "atom";
      } else {
        stream.eatWhile(/[-a-zA-Z]/);
        return "error";
      }
    } else if (ch == "~") {
      stream.next();
      ch = stream.peek();
      if (ch == "=") {
        stream.next();
        ch = stream.peek();
        if (ch == "=") {
          stream.next();
          return "operator";
        }
        return "operator";
      }
      return "operator";
    } else if (ch == ":") {
      stream.next();
      ch = stream.peek();
      if (ch == "=") {
        stream.next();
        return "operator";
      } else if (ch == ":") {
        stream.next();
        return "punctuation";
      }
    } else if ("[](){}".indexOf(ch) != -1) {
      stream.next();
      return "bracket";
    } else if (".,".indexOf(ch) != -1) {
      stream.next();
      return "punctuation";
    } else if (stream.match("end")) {
      return "keyword";
    }
    for (var name in patterns) {
      if (patterns.hasOwnProperty(name)) {
        var pattern = patterns[name];
        if ((pattern instanceof Array && some(pattern, function(p) {
          return stream.match(p);
        })) || stream.match(pattern))
          return patternStyles[name];
      }
    }
    if (/[+\-*\/^=<>&|]/.test(ch)) {
      stream.next();
      return "operator";
    }
    if (stream.match("define")) {
      return "def";
    } else {
      stream.eatWhile(/[\w\-]/);
      // Keyword
      if (wordLookup.hasOwnProperty(stream.current())) {
        return styleLookup[stream.current()];
      } else if (stream.current().match(symbol)) {
        return "variable";
      } else {
        stream.next();
        return "variable-2";
      }
    }
  }

  function tokenComment(stream, state) {
    var maybeEnd = false, maybeNested = false, nestedCount = 0, ch;
    while ((ch = stream.next())) {
      if (ch == "/" && maybeEnd) {
        if (nestedCount > 0) {
          nestedCount--;
        } else {
          state.tokenize = tokenBase;
          break;
        }
      } else if (ch == "*" && maybeNested) {
        nestedCount++;
      }
      maybeEnd = (ch == "*");
      maybeNested = (ch == "/");
    }
    return "comment";
  }

  function tokenString(quote, style) {
    return function(stream, state) {
      var escaped = false, next, end = false;
      while ((next = stream.next()) != null) {
        if (next == quote && !escaped) {
          end = true;
          break;
        }
        escaped = !escaped && next == "\\";
      }
      if (end || !escaped) {
        state.tokenize = tokenBase;
      }
      return style;
    };
  }

  // Interface
  return {
    startState: function() {
      return {
        tokenize: tokenBase,
        currentIndent: 0
      };
    },
    token: function(stream, state) {
      if (stream.eatSpace())
        return null;
      var style = state.tokenize(stream, state);
      return style;
    },
    blockCommentStart: "/*",
    blockCommentEnd: "*/",
    lineComment: "--"
  };
});

CodeMirror.defineMIME("text/x-livecodescript", "livecodescript");

});
